// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/lib/reg.c $
// $Rev: 32426 $
// $Date:$

#include <errno.h>

#include "dio40_lib.h"



/******************************************************************************
*
*	Function:	dio40_reg_mod
*
*	Purpose:
*
*		Perform a read-modify-write operation on a register.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		reg		This is the identifier for the register to access.
*
*		value	This is the value for the bits to modify.
*
*		mask	The set of bits to access. If a bit is set here, then the
*				register bit is accessed. If the bit is not set, then it is
*				unmodified.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_reg_mod(int fd, u32 reg, u32 value, u32 mask)
{
	gsc_reg_t	arg;
	int			errs;

	arg.reg		= reg;
	arg.value	= value;
	arg.mask	= mask;
	errs		= dio40_ioctl(fd, DIO40_IOCTL_REG_MOD, &arg);
	errs		= errs ? errno : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	dio40_reg_read
*
*	Purpose:
*
*		Read a value from a register.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		reg		This is the identifier for the register to access.
*
*		value	The value read is recorded here.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_reg_read(int fd, u32 reg, u32* value)
{
	gsc_reg_t	arg;
	int			errs;

	arg.reg		= reg;
	arg.value	= 0xDEADBEEF;
	arg.mask	= 0;
	errs		= dio40_ioctl(fd, DIO40_IOCTL_REG_READ, &arg);
	errs		= errs ? errno : 0;

	if (value)
		value[0]	= arg.value;

	return(errs);
}



/******************************************************************************
*
*	Function:	dio40_reg_write
*
*	Purpose:
*
*		Write a value to a register.
*
*	Arguments:
*
*		fd		The file descriptor of the device to access.
*
*		reg		This is the identifier for the register to access.
*
*		value	The value to write to the register.
*
*	Returned:
*
*		0		All went well.
*		else	The errno value for the error that occurrred.
*
******************************************************************************/

int dio40_reg_write(int fd, u32 reg, u32 value)
{
	gsc_reg_t	arg;
	int			errs;

	arg.reg		= reg;
	arg.value	= value;
	arg.mask	= 0;
	errs		= dio40_ioctl(fd, DIO40_IOCTL_REG_WRITE, &arg);
	errs		= errs ? errno : 0;
	return(errs);
}



