// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/sbtest/bsr.c $
// $Rev: 32587 $
// $Date:$

#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#include "main.h"



/******************************************************************************
*
*	Function:	_bsr_ciar_bit_test
*
*	Purpose:
*
*		Verify that the specified CIAR appear in BSR.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		index	The desired BSR bit.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _bsr_ciar_bit_test(int fd, int index)
{
	char	buf[32];
	u32		bsr;
	u32		ciar;
	int		errs	= 0;
	u32		rx;
	u32		tx;

	sprintf(buf, "BSR.D%d <- CIAR.D%d", index, index + 1);
	label(buf);

	rx		= 1 << index;
	tx		= 1 << (index + 1);

	// Configure the port as an output.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, tx);

	// Drive the output high and verify the input.
	errs	+= dio40_reg_write(fd, DIO40_GSC_COAR, tx);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CIAR, &ciar);
	errs	+= dio40_reg_read(fd, DIO40_GSC_BSR, &bsr);
	errs	+= (ciar & tx) ? 0 : 1;
	errs	+= (bsr & rx) ? 0 : 1;

	// Drive the output low and verify the input.
	errs	+= dio40_reg_write(fd, DIO40_GSC_COAR, 0);
	errs	+= dio40_reg_read(fd, DIO40_GSC_CIAR, &ciar);
	errs	+= dio40_reg_read(fd, DIO40_GSC_BSR, &bsr);
	errs	+= (ciar & tx) ? 1 : 0;
	errs	+= (bsr & rx) ? 1 : 0;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}


/******************************************************************************
*
*	Function:	_bsr_ciar_test
*
*	Purpose:
*
*		Verify that CIAR 1-7 appear in BSR 0-6.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _bsr_ciar_test(int fd)
{
	int	errs	= 0;
	int	i;

	label("BSR - CIAR");
	printf("\n");
	label_level_inc();

	for (i = 0; i < 7; i++)
		errs	+= _bsr_ciar_bit_test(fd, i);

	label_level_dec();
	return(errs);
}


/******************************************************************************
*
*	Function:	bsr_test
*
*	Purpose:
*
*		Perform BSR cable status tests.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int bsr_test(int fd)
{
	int	errs	= 0;

	label("BSR Status Tests");
	printf("\n");
	label_level_inc();

	errs	+= _bsr_ciar_test(fd);

	label_level_dec();
	return(errs);
}



