// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/sbtest/led.c $
// $Rev: 32587 $
// $Date:$

#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#include "main.h"



/******************************************************************************
*
*	Function:	_get_test
*
*	Purpose:
*
*		Perform a test of the dio40_led_get() library call.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _get_test(int fd)
{
	int	errs	= 0;
	int	i;
	u8	led;

	label("dio40_led_get");

	for (i = 0; i <= 255; i++)
	{
		errs	+= dio40_reg_write(fd, DIO40_GSC_LOR, i);
		errs	+= dio40_led_get(fd, &led);
		errs	+= (i == led) ? 0 : 1;
	}

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_mod_test
*
*	Purpose:
*
*		Perform a test of the dio40_mod_get() library call.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _mod_test(int fd)
{
	u8	bit;
	int	errs	= 0;
	int	i;
	u8	led;

	label("dio40_led_mod");
	errs	+= dio40_reg_write(fd, DIO40_GSC_LOR, 0xFF);

	for (i = 0; i < 8; i++)
	{
		bit		= 1 << i;

		// Turn it off and verify that the state of the LEDs.
		errs	+= dio40_led_mod(fd, 0, bit);
		errs	+= dio40_led_get(fd, &led);
		errs	+= (led & bit) ? 1 : 0;
		errs	+= (((led ^ bit) & 0xFF) != 0xFF) ? 1 : 0;

		// Turn it on and verify that the state of the LEDs.
		errs	+= dio40_led_mod(fd, bit, bit);
		errs	+= dio40_led_get(fd, &led);
		errs	+= (led & bit) ? 0 : 1;
		errs	+= ((led & 0xFF) != 0xFF) ? 1 : 0;
	}

	errs	+= dio40_reg_write(fd, DIO40_GSC_LOR, 0);
	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_set_test
*
*	Purpose:
*
*		Perform a test of the dio40_led_set() library call.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _set_test(int fd)
{
	int	errs	= 0;
	int	i;
	u32	lor;

	label("dio40_led_set");

	for (i = 0; i <= 255; i++)
	{
		errs	+= dio40_led_set(fd, i);
		errs	+= dio40_reg_read(fd, DIO40_GSC_LOR, &lor);
		lor		&= 0xFF;
		errs	+= (lor == i) ? 0 : 1;
	}

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_sweep_test
*
*	Purpose:
*
*		Perform a quick sweep test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _sweep_test(int fd)
{
	u32	bit;
	int	errs	= 0;
	int	i;

	label("Sweep");

	for (i = 1; i < 10; i++)
	{
		// Sweep from bottom to top.

		for (bit = 0x1; bit <= 0x80; bit *= 2)
		{
			errs	+= dio40_led_set(fd, bit);
			usleep(50000);
		}

		// Sweep from cwtop to bottom.

		for (bit = 0x100; bit >= 0x02;)
		{
			bit		/= 2;
			errs	+= dio40_led_set(fd, bit);
			usleep(50000);
		}
	}

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "DONE");
	return(errs);
}


/******************************************************************************
*
*	Function:	led_test
*
*	Purpose:
*
*		Perform an LED test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int led_test(int fd)
{
	int	errs	= 0;

	label("LED Test (board)");
	printf("\n");
	label_level_inc();

	errs	+= _sweep_test(fd);

	label_level_dec();

	label("LED Test (library)");
	printf("\n");
	label_level_inc();

	errs	+= _get_test(fd);
	errs	+= _mod_test(fd);
	errs	+= _set_test(fd);

	label_level_dec();
	return(errs);
}




