// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO40/legacy/DIO40_Linux_1.00.0_GSC_DN/sbtest/loopback.c $
// $Rev: 32587 $
// $Date:$

#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#include "main.h"



/******************************************************************************
*
*	Function:	_a_in_get_bit_test
*
*	Purpose:
*
*		Perform a test of the dio40_gpio_a_in_get() library call on the given
*		cable pin.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		index	The index of the bit/cable pin to test.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a_in_get_bit_test(int fd, int index)
{
	char	buf[32];
	u8		data;
	u32		dst_bit;
	int		dst_i;
	int		errs	= 0;
	u32		src_bit;
	int		src_i;

	dst_i	= index;
	src_i	= dst_i + ((index + 1) % 2);

	if (src_i == dst_i)
		src_i--;

	sprintf(buf, "D%d <- D%d", dst_i, src_i);
	label(buf);

	dst_bit	= 1UL << dst_i;
	src_bit	= 1UL << src_i;

	// Set the source to drive the destination via the loop back cable.
	errs	+= dio40_gpio_a_dir_set(fd, 0);
	errs	+= dio40_gpio_a_dir_mod(fd, src_bit, src_bit);

	// Drive the source high and verify the destination.
	errs	+= dio40_gpio_a_out_set(fd, src_bit);
	errs	+= dio40_gpio_a_in_get(fd, &data);
	errs	+= (data & dst_bit) ? 0 : 1;

	// Drive the source low and verify the destination.
	errs	+= dio40_gpio_a_out_set(fd, 0);
	errs	+= dio40_gpio_a_in_get(fd, &data);
	errs	+= (data & dst_bit) ? 1 : 0;

	// Reset things.
	errs	+= dio40_reset(fd);

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_a_in_get_test
*
*	Purpose:
*
*		Perform a test of the dio40_gpio_a_in_get() library call.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _a_in_get_test(int fd)
{
	int	errs	= 0;
	int	i;

	label("dio40_gpio_a_in_get");
	printf("\n");
	label_level_inc();

	for (i = 0; i < 8; i++)
		errs	+= _a_in_get_bit_test(fd, i);

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_b_in_get_bit_test
*
*	Purpose:
*
*		Perform a test of the dio40_gpio_b_in_get() library call on the given
*		cable pin.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		index	The index of the bit/cable pin to test.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _b_in_get_bit_test(int fd, int index)
{
	char	buf[32];
	u32		data;
	u32		dst_bit;
	int		dst_i;
	int		errs	= 0;
	u32		src_bit;
	int		src_i;

	dst_i	= index;
	src_i	= dst_i + ((index + 1) % 2);

	if (src_i == dst_i)
		src_i--;

	sprintf(buf, "D%d <- D%d", dst_i, src_i);
	label(buf);

	dst_bit	= 1UL << dst_i;
	src_bit	= 1UL << src_i;

	// Set the source to drive the destination via the loop back cable.
	errs	+= dio40_gpio_b_dir_set(fd, 0);
	errs	+= dio40_gpio_b_dir_mod(fd, src_bit, src_bit);

	// Drive the source high and verify the destination.
	errs	+= dio40_gpio_b_out_set(fd, src_bit);
	errs	+= dio40_gpio_b_in_get(fd, &data);
	errs	+= (data & dst_bit) ? 0 : 1;

	// Drive the source low and verify the destination.
	errs	+= dio40_gpio_b_out_set(fd, 0);
	errs	+= dio40_gpio_b_in_get(fd, &data);
	errs	+= (data & dst_bit) ? 1 : 0;

	// Reset things.
	errs	+= dio40_reset(fd);

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_b_in_get_test
*
*	Purpose:
*
*		Perform a test of the dio40_gpio_b_in_get() library call.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _b_in_get_test(int fd)
{
	int	errs	= 0;
	int	i;

	label("dio40_gpio_b_in_get");
	printf("\n");
	label_level_inc();

	for (i = 0; i < 32; i++)
		errs	+= _b_in_get_bit_test(fd, i);

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_loopback_bit_test
*
*	Purpose:
*
*		Perform loop back tests on the given registeri bit.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		ctrl	The identifier of the control register.
*
*		output	The identifier for the output register.
*
*		input	The identifier for the input register.
*
*		tx		Write data to this index bit.
*
*		rx		Read data from this index bit.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _loopback_bit_test(
	int	fd,
	u32	ctrl,
	u32	output,
	u32	input,
	int	tx,
	int	rx)
{
	char	buf[32];
	int		errs	= 0;
	u32		reg;
	u32		rx_bit;
	u32		tx_bit;

	sprintf(buf, "D%d -> D%d", tx, rx);
	label(buf);

	rx_bit	= 1UL << rx;
	tx_bit	= 1UL << tx;
	// Set everything as inputs.
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, 0);
	errs	+= dio40_reg_write(fd, DIO40_GSC_CCBR, 0);

	// Set all outputs low.
	errs	+= dio40_reg_write(fd, DIO40_GSC_COAR, 0);
	errs	+= dio40_reg_write(fd, DIO40_GSC_COBR, 0);

	// Adjust the direction, set the output LOW, get and check the input.
	errs	+= dio40_reg_mod(fd, ctrl, tx_bit, tx_bit);
	errs	+= dio40_reg_mod(fd, output, 0, tx_bit);
	errs	+= dio40_reg_read(fd, input, &reg);
	errs	+= (reg & rx_bit) ? 1 : 0;
	errs	+= (reg & tx_bit) ? 1 : 0;

	// Adjust the direction, set the output HIGH, get and check the input.
	errs	+= dio40_reg_mod(fd, ctrl, tx_bit, tx_bit);
	errs	+= dio40_reg_mod(fd, output, tx_bit, tx_bit);
	errs	+= dio40_reg_read(fd, input, &reg);
	errs	+= (reg & rx_bit) ? 0 : 1;
	errs	+= (reg & tx_bit) ? 0 : 1;

	// Do a reset and verify that the direction has been reset to an input.
	errs	+= dio40_reset(fd);
	errs	+= dio40_reg_read(fd, ctrl, &reg);
	errs	+= (reg & rx_bit) ? 1 : 0;
	errs	+= dio40_reg_read(fd, output, &reg);
	errs	+= (reg & tx_bit) ? 1 : 0;

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_loopback_tx_clock_test
*
*	Purpose:
*
*		Perform a Tx Clock loop back test on the given register bit.
*		This is a very simple test that looks only for both high an low states.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		tx		Write data to this index bit.
*
*		rx		Read data from this index bit.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _loopback_tx_clock_test(int fd, int tx, int rx)
{
	char	buf[32];
	u32		ccar;
	u32		ciar;
	int		errs	= 0;
	long	high	= 0;
	long	l;
	long	low		= 0;
	u32		rx_bit;
	u32		tx_bit;

	sprintf(buf, "D%d -> D%d (Tx Clock)", tx, rx);
	label(buf);

	rx_bit	= 1UL << rx;
	tx_bit	= 1UL << tx;

	// Configure the port.

	if (tx == 0)
		ccar	= tx_bit | 0x00100;
	else
		ccar	= tx_bit | 0x10000;

	errs	+= dio40_reg_write(fd, DIO40_GSC_CCAR, ccar);
	errs	+= dio40_reg_write(fd, DIO40_GSC_COAR, 0);

	// Read the input counting levels.

	for (l = 0; l < 1000000; l++)
	{
		errs	+= dio40_reg_read(fd, DIO40_GSC_CIAR, &ciar);

		if (ciar & rx_bit)
			high++;
		else
			low++;
	}

	errs	+= ((high == 0) || (low == 0)) ? 1 : 0;

	// Do a reset.
	errs	+= dio40_reset(fd);

	errs	= errs ? 1 : 0;
	printf("%s\n", errs ? "FAIL <---" : "PASS");
	return(errs);
}



/******************************************************************************
*
*	Function:	_loopback_reg_test
*
*	Purpose:
*
*		Perform loop back tests on the given register.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		ext		Is the external loopback cable attached? If it is we'll
*				do additional testing.
*
*		name	The name of the port group.
*
*		ctrl	The identifier of the control register.
*
*		output	The identifier for the output register.
*
*		input	The identifier for the input register.
*
*		bits	The number of outputs in the port.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _loopback_reg_test(
	int			fd,
	int			ext,
	const char*	name,
	u32			ctrl,
	u32			output,
	u32			input,
	int			bits)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	int		rx;
	int		tx;

	sprintf(buf, "%s (D0 - D%d)", name, bits - 1);
	label(buf);
	printf("\n");
	label_level_inc();

	label("Internal Loop Back");
	printf("\n");
	label_level_inc();

	for (i = 0; i < bits; i++)
	{
		errs	+= _loopback_bit_test(fd, ctrl, output, input, i, i);

		if (ctrl == DIO40_GSC_CCBR)
			;
		else if (i == 0)
			errs	+= _loopback_tx_clock_test(fd, i, i);
		else if (i == 7)
			errs	+= _loopback_tx_clock_test(fd, i, i);
	}

	label_level_dec();

	label("External Loop Back");

	if (ext == 0)
	{
		printf("SKIPPED (cable not attached)\n");
	}
	else
	{
		printf("\n");
		label_level_inc();

		for (i = 0; i < bits; i++)
		{
			tx	= i;
			rx	= tx + ((i + 1) % 2);

			if (tx == rx)
				rx--;

			errs	+= _loopback_bit_test(fd, ctrl, output, input, tx, rx);

			if (ctrl == DIO40_GSC_CCBR)
				;
			else if (i == 0)
				errs	+= _loopback_tx_clock_test(fd, tx, rx);
			else if (i == 7)
				errs	+= _loopback_tx_clock_test(fd, tx, rx);
		}

		label_level_dec();
	}

	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	loopback_test
*
*	Purpose:
*
*		Perform loopback tests.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		ext		Is the external loopback cable attached? If it is we'll
*				do additional testing.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int loopback_test(int fd, int ext)
{
	int	errs	= 0;

	label("Loop Back Tests (board)");
	printf("\n");
	label_level_inc();

	errs	+= _loopback_reg_test(	fd,
									ext,
									"Port A",
									DIO40_GSC_CCAR,
									DIO40_GSC_COAR,
									DIO40_GSC_CIAR,
									8);
	errs	+= _loopback_reg_test(	fd,
									ext,
									"Port B",
									DIO40_GSC_CCBR,
									DIO40_GSC_COBR,
									DIO40_GSC_CIBR,
									32);

	label_level_dec();

	label("Loop Back Tests (library)");

	if (ext == 0)
	{
		printf("SKIPPED (cable not attached)\n");
	}
	else
	{
		printf("\n");
		label_level_inc();

		errs	+= _a_in_get_test(fd);
		errs	+= _b_in_get_test(fd);

		label_level_dec();
	}

	return(errs);
}



